#include <unistd.h>
#include <limits.h>
#include "lzma/IInOutStreams.h"
#include "lzma/LZMAEncoder.h"
#include "lzma/LZMADecoder.h"
#include "7zwrapper.h"


NCompress::NLZMA::CDecoder *decoder = NULL;

void inflateLZMAInit(void)
{
	decoder = new NCompress::NLZMA::CDecoder;
}

int inflateLZMA(void* in_data, int in_size, void* out_data, int out_size, int* total_out)
{
	
	ISequentialInStream  inStream((char*)in_data,(unsigned)in_size);
	ISequentialOutStream outStream((char *)out_data,(unsigned)out_size);
	
	if(!decoder){
		return LZMA_ERR;
	}
	
	try {
		if(decoder->ReadCoderProperties(&inStream) != S_OK){
			return LZMA_ERR;
		}
	    
		if(decoder->Code(&inStream, &outStream) != S_OK){
			return LZMA_ERR;
		}
		*total_out = (int) outStream.size_get();
		return LZMA_OK;
	}
	catch (...) {
		return LZMA_ERR;
	}
	
}

void inflateLZMAEnd(void)
{
	delete decoder;
	decoder = NULL;
}

int compressLZMA(void *in_data,  unsigned long in_size, void* out_data, unsigned long* total_out)
{
	NCompress::NLZMA::CEncoder encoder;
	ISequentialInStream  inStream((char*)in_data,(unsigned)in_size);
	/* writing to an mmaped file: we don't have output size limit */
	ISequentialOutStream outStream((char *)out_data, UINT_MAX); 
	unsigned int algo = 1;
	unsigned int dictionary_size = 1 << 20;
	unsigned int num_fast_bytes = 32;
	try {
		if (encoder.SetDictionarySize(dictionary_size) != S_OK)
			return LZMA_ERR;
		
		if (encoder.SetEncoderNumFastBytes(num_fast_bytes) != S_OK)
			return LZMA_ERR;
		
		if (encoder.SetEncoderAlgorithm(algo) != S_OK)
		return LZMA_ERR;
						
		if (encoder.WriteCoderProperties(&outStream) != S_OK)
			return LZMA_ERR;
		
		if (encoder.Code(&inStream, &outStream) != S_OK)
			return LZMA_ERR;
		*total_out = (unsigned long) outStream.size_get();
		return LZMA_OK;
	} catch (...) {
		return LZMA_ERR;
	}
}
/*
 * Local variables:
 * c-file-style: "linux"
 * End:
 */
